@@ -465,6 +465,11 @@ void Fragment::AddItem ()
 		m_Atom->SetChargeItem (text);
 	} else
 		m_Atom->SetChargeItem (NULL);
+	// update the size
+	gccv::Rect rect;
+	m_TextItem->GetBounds (&rect, NULL);
+	m_length = rect.x1 - rect.x0;
+	m_height = rect.y1 - rect.y0;
 }
 
 void Fragment::UpdateItem ()
@@ -683,6 +688,7 @@ bool Fragment::Load (xmlNodePtr node)
 	char* tmp;
 	double size = (double) pTheme->GetFontSize () / PANGO_SCALE;
 	gccv::TextTag *tag;
+	m_Atom->SetId ("a0"); // avoids confusion with another atom while loading
 	while (child) {
 		if (!strcmp ((const char*) child->name, "text")) {
 			tmp = (char*) xmlNodeGetContent (child);
@@ -690,13 +696,13 @@ bool Fragment::Load (xmlNodePtr node)
 				m_buf += tmp;
 			xmlFree (tmp);
 		} else if (!strcmp ((const char*) child->name, "atom")) {
+			AddChild (m_Atom);
 			if (!m_Atom->Load (child))
 				return false;
 			m_BeginAtom = m_buf.length ();
 			m_buf += m_Atom->GetSymbol();
 			m_Atom->SetCoords (m_x, m_y);
 			m_EndAtom = m_buf.length ();
-			AddChild (m_Atom);
 		} else if (!strcmp ((const char*) child->name, "residue")) {
 			// replace the atom by a residue
 			map<gcu::Atom*, gcu::Bond*>::iterator i;
@@ -789,22 +795,24 @@ bool Fragment::Load (xmlNodePtr node)
 		child = child->next;
 	}
 	if (!m_TextItem) {
-		// analyse text between tags to support previous versions
-		list <gccv::TextTag *> tags;
-		gccv::TextTagList::iterator i, iend = m_TagList.end ();
-		unsigned start = 0, end;
-		for (i = m_TagList.begin (); i != iend; i++)
-			tags.push_back (*i);
-		list <gccv::TextTag *>::iterator j, jend = tags.end ();
-		for (j = tags.begin (); j != jend; j++) {
-			end = (*j)->GetStartIndex ();
+		if ( static_cast <Document *> (GetDocument ())->GetSoftwareVersion () < 11000) {
+			// analyse text between tags to support previous versions
+			list <gccv::TextTag *> tags;
+			gccv::TextTagList::iterator i, iend = m_TagList.end ();
+			unsigned start = 0, end;
+			for (i = m_TagList.begin (); i != iend; i++)
+				tags.push_back (*i);
+			list <gccv::TextTag *>::iterator j, jend = tags.end ();
+			for (j = tags.begin (); j != jend; j++) {
+				end = (*j)->GetStartIndex ();
+				if (end > start)
+					AnalContent (start, end);
+				start = (*j)->GetEndIndex ();
+			}
+			end = m_buf.length ();
 			if (end > start)
 				AnalContent (start, end);
-			start = (*j)->GetEndIndex ();
 		}
-		end = m_buf.length ();
-		if (end > start)
-			AnalContent (start, end);
 	} else {
 		m_TextItem->SetText (m_buf);
 		while (!m_TagList.empty ()) {
@@ -1039,9 +1047,12 @@ void Fragment::AnalContent (unsigned start, unsigned &end)
 				default: // should not happen
 					break;
 				}
-				m_TextItem->ReplaceText (glyph, next, l);
-				m_buf = m_TextItem->GetText ();
-				text = m_buf.c_str ();
+				if (m_TextItem) {
+					m_TextItem->ReplaceText (glyph, next, l);
+					m_buf = m_TextItem->GetText ();
+					text = m_buf.c_str ();
+				} else
+					m_buf.replace (next, l, glyph);
 				end += glyph.length () - l;
 				start = next;
 			}
@@ -1065,8 +1076,6 @@ Object* Fragment::GetAtomAt (double x, double y, G_GNUC_UNUSED double z)
 		return NULL;
 	if (m_Atom->GetBondsNumber () || m_Atom->GetCharge ())
 		return m_Atom;
-	if (!pDoc)
-		return NULL;
 	double x0, y0;
 	x0 = (x - m_x) * pTheme->GetZoomFactor () + m_lbearing;
 	y0 = (y - m_y) * pTheme->GetZoomFactor () + m_ascent;
@@ -1075,6 +1084,8 @@ Object* Fragment::GetAtomAt (double x, double y, G_GNUC_UNUSED double z)
 	unsigned index, trailing;
 	int cur;
 	index = m_TextItem->GetIndexAt (x0, y0);
+	if (index > 0)
+		index--;
 	char c = m_buf[index];
 	cur = index;
 	while ((c >= 'a') && (c <= 'z') && cur >= 0) {
