@@ -4,7 +4,7 @@
  * GChemPaint library
  * mesomer.cc
  *
- * Copyright (C) 2005-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2005-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -27,6 +27,9 @@
 #include "mesomery.h"
 #include "molecule.h"
 #include "document.h"
+#include "mechanism-arrow.h"
+#include "view.h"
+#include "widgetdata.h"
 #include <glib/gi18n-lib.h>
 
 using namespace gcu;
@@ -36,7 +39,7 @@ namespace gcp {
 
 TypeId MesomerType;
 
-Mesomer::Mesomer (): Object (MesomerType)
+Mesomer::Mesomer (): MechanismStep (MesomerType)
 {
 }
 
@@ -51,15 +54,50 @@ Mesomer::~Mesomer ()
 		return;
 	map<string, Object *>::iterator i;
 	Object *Child, *Group = ms->GetGroup ();
+	list <MechanismArrow *> arrows;
 	while (HasChildren ()) {
 		Child = GetFirstChild (i);
 		GetParent ()->GetParent ()->AddChild (Child);
+		if (Child->GetType () == MechanismArrowType) {
+			arrows.push_back (static_cast <MechanismArrow *> (Child));
+			continue;
+		}
 		if (pOp && !Group)
 			pOp->AddObject (Child, 1);
 	}
+	while (!arrows.empty ()) {
+		MechanismArrow *arrow = arrows.front ();
+		MechanismStep *step;
+		Object *obj = arrow->GetSource (), *molecule = obj->GetMolecule (), *parent = molecule->GetParent ();
+		if (parent->GetType () == MechanismStepType) {
+			step = static_cast <MechanismStep *> (parent);
+			step->AddChild (arrow);
+		} else {
+			step = new MechanismStep ();
+			step->SetParent (parent);
+			step->AddChild (arrow);
+			step->AddChild (molecule);
+		}
+		obj = arrow->GetTarget ();
+		molecule = obj->GetMolecule ();
+		parent = molecule->GetParent ();
+		if (parent != step) {
+			if (parent->GetType () == MechanismStepType) {
+				map <string, Object *>::iterator it;
+				obj = parent->GetFirstChild (it);
+				while (obj) {
+					step->AddChild (obj);
+					obj = parent->GetFirstChild (it);
+				}
+				
+			} else
+				step->AddChild (molecule);
+		}
+		arrows.pop_front ();
+	}
 }
 
-Mesomer::Mesomer (Mesomery* mesomery, Molecule* molecule) throw (std::invalid_argument): Object (MesomerType)
+Mesomer::Mesomer (Mesomery* mesomery, Molecule* molecule) throw (std::invalid_argument): MechanismStep (MesomerType)
 {
 	if (!mesomery || !molecule)
 		throw invalid_argument ("NULL argument to Mesomer constructor!");
@@ -70,6 +108,21 @@ Mesomer::Mesomer (Mesomery* mesomery, Molecule* molecule) throw (std::invalid_ar
 	m_Molecule = molecule;
 }
 
+Mesomer::Mesomer (Mesomery *mesomery, MechanismStep *step) throw (std::invalid_argument): MechanismStep (MesomerType)
+{
+	if (!mesomery || !step)
+		throw invalid_argument (_("NULL argument to Mesomer constructor!"));
+	SetId ("ms1");
+	mesomery->AddChild (this);
+	GetDocument ()->EmptyTranslationTable();
+	map <string, Object *>::iterator it;
+	while (Object *obj = step->GetFirstChild (it)) {
+		if (obj->GetType () == MoleculeType)
+			m_Molecule = static_cast <Molecule *> (obj);
+		AddChild (obj);
+	}
+}
+
 void Mesomer::AddArrow (MesomeryArrow *arrow, Mesomer *mesomer) throw (std::invalid_argument)
 {
 	if (m_Arrows[mesomer])
@@ -99,13 +152,21 @@ bool Mesomer::OnSignal (G_GNUC_UNUSED SignalId Signal, G_GNUC_UNUSED Object *Chi
 bool Mesomer::Load (xmlNodePtr node)
 {
 	if (Object::Load (node)) {
-		if (GetChildrenNumber () != 1)
-			return false;
 		map<string, Object*>::iterator i;
-		m_Molecule = reinterpret_cast<Molecule *> (GetFirstChild (i));
+		Object *obj = GetFirstChild (i);
+		while (obj && obj->GetType () != MoleculeType)
+			obj = GetNextChild (i);
+		if (!obj)
+			return false;
+		m_Molecule = reinterpret_cast <Molecule *> (obj);
 		return true;
 	}
 	return false;
 }
 
+std::string Mesomer::Name ()
+{
+	return _("Mesomer");
+}
+
 }	//	namespace gcp
